<?php
/**
 * Diglin
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 *
 * @category    Diglin
 * @package     Onlinebiz_Optimize
 * @copyright   Copyright (c) 2011-2017 Diglin (http://www.onlinebiz.com)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
//set_include_path(BP . DS . 'lib' . DS . 'MatthiasMullie'. DS. 'Minify' . PS . get_include_path());
//require_once 'MatthiasMullie' . DS . 'Minify' . DS . 'JS.php';
//require_once 'MatthiasMullie' . DS . 'Minify' . DS . 'CSS.php';
use MatthiasMullie\Minify\CSS;
use MatthiasMullie\Minify\JS;

class Onlinebiz_Optimize_Block_Optimize_Head extends Onlinebiz_Optimize_Block_Optimize_Head_Abstract
{

  /**
   * Get HEAD HTML with CSS/JS/RSS definitions
   * (actually it also renders other elements, TODO: fix it up or rename this method)
   *
   * @return string
   */
  public function getCssJsHtml()
  {
    // separate items by types
    $lines = array();
    foreach ($this->_data['items'] as $item) {
      if (!is_null($item['cond']) && !$this->getData($item['cond']) || !isset($item['name'])) {
        continue;
      }
      $if = !empty($item['if']) ? $item['if'] : '';
      $params = !empty($item['params']) ? $item['params'] : '';
      switch ($item['type']) {
        case 'js':        // js/*.js
        case 'skin_js':   // skin/*/*.js
        case 'js_css':    // js/*.css
        case 'skin_css':  // skin/*/*.css
          $lines[$if][$item['type']][$params][$item['name']] = $item['name'];
          break;
        default:
          $this->_separateOtherHtmlHeadElements($lines, $if, $item['type'], $params, $item['name'], $item);
          break;
      }
    }

    // prepare HTML
    $shouldMergeJs = Mage::getStoreConfigFlag('dev/js/merge_files');
    $shouldMergeCss = Mage::getStoreConfigFlag('dev/css/merge_css_files');
    $html = '';
    foreach ($lines as $if => $items) {
      if (empty($items)) {
        continue;
      }
      if (!empty($if)) {
        // open !IE conditional using raw value
        if (strpos($if, "><!-->") !== false) {
          $html .= $if . "\n";
        } else {
          $html .= '<!--[if ' . $if . ']>' . "\n";
        }
      }

      $cssPreload = $this->_helper()->getGeneralConfig('css_preload');

      // static and skin css
      $html .= $this->_prepareStaticAndSkinElements('<link rel="%s" type="text/css" href="%s"%s />' . "\n",
        empty($items['js_css']) ? array() : $items['js_css'],
        empty($items['skin_css']) ? array() : $items['skin_css'],
        $shouldMergeCss ? array(Mage::getDesign(), 'getMergedCssUrl') : null
      );

      // static and skin javascripts
      $html .= $this->_prepareStaticAndSkinElements('<script type="text/javascript"%s src="%s"%s></script>' . "\n",
        empty($items['js']) ? array() : $items['js'],
        empty($items['skin_js']) ? array() : $items['skin_js'],
        $shouldMergeJs ? array(Mage::getDesign(), 'getMergedJsUrl') : null
      );

      // other stuff
      if (!empty($items['other'])) {
        $html .= $this->_prepareOtherHtmlHeadElements($items['other']) . "\n";
      }

      if (!empty($if)) {
        // close !IE conditional comments correctly
        if (strpos($if, "><!-->") !== false) {
          $html .= '<!--<![endif]-->' . "\n";
        } else {
          $html .= '<![endif]-->' . "\n";
        }
      }
    }

    if($cssPreload){
      $html .= '<script type="text/javascript">' . Onlinebiz_Optimize_Model_Css_Observer::LOADCSS. '</script>';
    }
    return $html;
  }
  

  /**
   * Merge static and skin files of the same format into a set of HEAD directives or even into a directive
   *
   * Will attempt to merge into 1 directive, if merging callback is provided. In this case it will generate
   * filenames, rather than render urls.
   * The merger callback is responsible for checking whether files exist, merging them and giving result URL
   *
   * @param string $format - HTML element format for sprintf('<element src="%s"%s />', $src, $params)
   * @param array $staticItems - array of relative names of static items to be grabbed from js/ folder
   * @param array $skinItems - array of relative names of skin items to be found in skins according to design config
   * @param callback $mergeCallback
   * @return string
   */
  protected function &_prepareStaticAndSkinElements($format, array $staticItems, array $skinItems, $mergeCallback = null)
  {
    $designPackage = Mage::getDesign();

    $baseJsUrl = Mage::getBaseUrl('js');
    $items = array();
    if ($mergeCallback && !is_callable($mergeCallback)) {
      $mergeCallback = null;
    }

    // get static files from the js folder, no need in lookups
    foreach ($staticItems as $params => $rows) {
      foreach ($rows as $name) {
        if (strpos($name, '.css') !== false && $this->_helper()->isUseCompression()) {
          $items[$params][] = $this->_prepareCssCompression($name, $mergeCallback, true);
        } else if (strpos($name, '.js') !== false && $this->_helper()->isUseCompression('js')) {
          $items[$params][] = $this->_prepareJsCompression($name, $mergeCallback, true);
        } else {
          $items[$params][] = $mergeCallback ? Mage::getBaseDir() . DS . 'js' . DS . $name : $baseJsUrl . $name;
        }
      }
    }

    // lookup each file basing on current theme configuration
    foreach ($skinItems as $params => $rows) {
      foreach ($rows as $name) {
        if (strpos($name, '.css') !== false && $this->_helper()->isUseCompression()) {
          if ($result = $this->_prepareCssCompression($name, $mergeCallback))
            $items[$params][] = $result;
        } else if (strpos($name, '.js') !== false && $this->_helper()->isUseCompression('js')) {
          if ($result = $this->_prepareJsCompression($name, $mergeCallback)) ;
          $items[$params][] = $result;
        } else {
          $items[$params][] = $mergeCallback ? $designPackage->getFilename($name, array('_type' => 'skin'))
            : $designPackage->getSkinUrl($name, array());
        }
      }
    }

    $html = '';

    // Change postion for cirtical files;
    if (isset($items['cirtical']) && !empty($items['cirtical'])){
      $criticalCss = array();
      $criticalCss['cirtical'] = $items['cirtical'];
      unset($items['cirtical']);
      $items = array_merge($criticalCss, $items);
    }

    foreach ($items as $params => $rows) {
      // attempt to merge
      $mergedUrl = false;
      if ($mergeCallback) {
        $mergedUrl = call_user_func($mergeCallback, $rows);
      }
      // render elements
      $params = trim($params);
      $params = $params ? ' ' . $params : '';
      $usePreload = $this->_helper()->getGeneralConfig('css_preload');
      $isCss = (strpos($format, 'type="text/css"') !== false) ? true : false;
      $cssRel = $usePreload ? 'preload' : 'stylesheet';
      if (!$isCss){
        $cssRel = '';
      }
      $preload = ($usePreload && $isCss) ? ' as="style" onload="this.onload=null;this.rel=\'stylesheet\'"' : '';
      $includeCss = $this->_helper()->getGeneralConfig('critical');
      if ($mergedUrl) {
        if ($isCss && $includeCss && trim($params) == 'cirtical' && !$this->_helper()->isUrlsBlacklist()) {
          $filename = Zend_Uri::factory($mergedUrl)->getPath();
          $html .= $this->getContentCirticalStyle($filename);
          if (!$html) {
            $html .= sprintf($format, $cssRel, $mergedUrl, $params . $preload);
            if ($usePreload){
              $cssRel = 'stylesheet';
              $html .= '<noscript>' . str_replace( PHP_EOL,'', sprintf($format, $cssRel, $mergedUrl, $params)) .'</noscript>' ."\n";
            }
          }
        } else {
          $html .= sprintf($format, $cssRel, $mergedUrl, $params . $preload);
          if ($usePreload && $isCss){
            $cssRel = 'stylesheet';
            $html .= '<noscript>' . str_replace( PHP_EOL,'', sprintf($format, $cssRel, $mergedUrl, $params)) .'</noscript>' ."\n";
          }
        }
      } else {
        foreach ($rows as $src) {
          if ($isCss && $includeCss && trim($params) == 'cirtical' && !$this->_helper()->isUrlsBlacklist()) {
            $filename = Zend_Uri::factory($src)->getPath();
            $html .= $this->getContentCirticalStyle($filename);
            if (!$html) {
              if ($usePreload){
                $cssRel = 'stylesheet';
                $html .= '<noscript>' . str_replace( PHP_EOL,'', sprintf($format, $cssRel, $src, $params)) .'</noscript>' ."\n";
              }
            }
          } else {
            $html .= sprintf($format, $cssRel, $src, $params . $preload);
            if ($usePreload && $isCss){
              $cssRel = 'stylesheet';
              $html .= '<noscript>' . str_replace( PHP_EOL,'', sprintf($format, $cssRel, $src, $params)) .'</noscript>' ."\n";
            }
          }
        }
      }
    }
    return $html;
  }

  protected function _helper()
  {
    return Mage::helper('onlinebiz_optimize');
  }

  /**
   * Compress CSS files and write them to media/css folders
   *
   * @param string $name
   * @param string|array $mergeCallback
   * @param bool $static
   * @return string
   */
  protected function _prepareCssCompression($name, $mergeCallback = null, $static = false)
  {
    $info = $this->_helper()->getCompressedInfo($name, 'css', $static);
    if (!is_array($info))
      return;
    if (!isset($info['result']) || $info['result'] != false) {
      $info['result'] = true;
    }

    if (!file_exists($info['targetPathFile']) && $info['result'] ||
      $info['result'] && filemtime($info['orgskin_path']) > filemtime($info['targetPathFile'])) {
      $minifyCSS = new CSS($info['orgskin_path']);
      $info['result'] = $minifyCSS->minify($info['targetPathFile']);
    };
    return $this->_helper()->getResultPath($info, $mergeCallback);
  }

  /**
   * Compress JS files and write them to media/js folders
   *
   * @param string $name
   * @param string|array $mergeCallback
   * @param bool $static
   * @return string
   */
  protected function _prepareJsCompression($name, $mergeCallback = null, $static = false)
  {
    $info = $this->_helper()->getCompressedInfo($name, 'js', $static);
    if (!isset($info['result']) || $info['result'] != false) {
      $info['result'] = true;
    }

    if (!file_exists($info['targetPathFile']) && $info['result'] || $info['result'] && filemtime($info['orgskin_path']) > filemtime($info['targetPathFile'])) {
      $minifyJS = new JS($info['orgskin_path']);
      $info['result'] = $minifyJS->minify($info['targetPathFile']);
    }
    return $this->_helper()->getResultPath($info, $mergeCallback);
  }

  protected function getContentCirticalStyle($filename)
  {
    $fileDir = Mage::getBaseDir() . $filename;
    try {
      if (file_exists($fileDir)) {
        return '<style type="text/css">' . @file_get_contents($fileDir) . '</style>' ."\n";
      } else {
        return null;
      }

    } catch (Exception $e) {
      return null;
    }
  }

}
