<?php

class Onlinebiz_Optimize_Model_Core_Design_Package extends Mage_Core_Model_Design_Package
{
	protected function _helper()
	{
		return Mage::helper('onlinebiz_optimize');
	}

	protected function isCloud()
	{
		return $this->_helper()->isCloudinaryActive($this->getStore());
	}

	/**
	* Get skin file url
	*
	* @param string $file
	* @param array $params
	* @return string
	*/
	public function getSkinUrl($file = null, array $params = array())
	{
		Varien_Profiler::start(__METHOD__);
		if (empty($params['_type'])) {
			$params['_type'] = 'skin';
		}
		if (empty($params['_default'])) {
			$params['_default'] = false;
		}
		$this->updateParamDefaults($params);
		if (!empty($file)) {
			if ($this->_helper()->checkVersion('1.9')) {
				$result = $this->_fallback(
					$file,
					$params,
					$this->_fallback->getFallbackScheme(
						$params['_area'],
						$params['_package'],
						$params['_theme']
						)
				);
			} else {
				$result = $this->_fallback(
					$file,
					$params,
					array(
						array(),
						array('_theme' => $this->getFallbackTheme()),
						array('_theme' => self::DEFAULT_THEME),
					)
				);
			}
		}

		if($this->isNotImageFile($file))
			$result = $this->getSkinJsCss($file, $params) . (empty($file) ? '' : $file);
		else
			$result = $this->getSkinBaseUrl($params) . (empty($file) ? '' : $file . $this->_helper()->getIMGixOptions());

		Varien_Profiler::stop(__METHOD__);
		return $result;
	}

	/**
	* Get skin file url
	*
	* @param string $file
	* @param array $params
	* @return string
	*/
	public function getSkinMageUrl($file = null, array $params = array())
	{
		Varien_Profiler::start(__METHOD__);
		if (empty($params['_type'])) {
			$params['_type'] = 'skin';
		}
		if (empty($params['_default'])) {
			$params['_default'] = false;
		}
		$this->updateParamDefaults($params);
		if (!empty($file)) {
			$result = $this->_fallback($file, $params, array(
				array(),
				array('_theme' => $this->getFallbackTheme()),
				array('_theme' => self::DEFAULT_THEME),
			));
		}
		$result = $this->getSkinBaseMageUrl($params) . (empty($file) ? '' : $file);
		Varien_Profiler::stop(__METHOD__);
		return $result;
	}


	public function getSkinJsCss($file = null, array $params=array())
	{
		$skinMergeURL = Mage::getBaseUrl(Onlinebiz_Optimize_Model_Core_Store::URL_TYPE_SKIN_MERGE);
		$params['_type'] = 'skin';
		$this->updateParamDefaults($params);
		$baseUrl = $skinMergeURL . $params['_area'] . '/' . $params['_package'] . '/' . $params['_theme'] . '/';
		return $baseUrl;
	}

	public function isNotImageFile($file) {
		$info = pathinfo($file);
		$extension = '';
		if(isset($info['extension']))
		$extension = $info['extension'];
		return in_array(strtolower($extension), array("css", "js"));
	}


	/**
	* Merge specified css files and return URL to the merged file on success
	*
	* @param $files
	* @return string
	*/
	public function getMergedCssUrl($files)
	{
		// secure or unsecure
		$isSecure  = Mage::app()->getRequest()->isSecure();
		$mergerDir = $isSecure ? 'css_secure' : 'css';
		$targetDir = $this->_initMergerDir($mergerDir);
		if (!$targetDir) {
			return '';
		}

		// base hostname & port
		$baseMediaUrl = Mage::getBaseUrl('media', $isSecure);
		$hostname = parse_url($baseMediaUrl, PHP_URL_HOST);
		$port = parse_url($baseMediaUrl, PHP_URL_PORT);
		if (false === $port) {
			$port = $isSecure ? 443 : 80;
		}

		// merge into target file
		$targetFilename = md5(implode(',', $files) . "|{$hostname}|{$port}") . '.min.css';
		$localFiles 	= $this->_removeParamFiles($files);
		$mergeFilesResult = $this->_mergeFiles(
			$localFiles, $targetDir . DS . $targetFilename,
			false,
			array($this, 'beforeMergeCss'),
			'css'
		);

		// get Merge Media URL
		$mediaMerge = $this->_helper()->getMediaMergeUrl($mergerDir . DS . $targetFilename, $this->getStore());
		$version = $this->_helper()->_getFileVersion($mergerDir . '/' . $targetFilename);
		if(!empty($mediaMerge)){
			return $mediaMerge . $mergerDir . '/' . $targetFilename . $version;
		}

		if ($mergeFilesResult) {
			return $baseMediaUrl . $mergerDir . '/' . $targetFilename . $version;
		}
		return '';
	}


	/**
	* Merge specified javascript files and return URL to the merged file on success
	*
	* @param $files
	* @return string
	*/
	public function getMergedJsUrl($files)
	{
		$targetFilename = md5(implode(',', $files)) . '.min.js';
		$targetDir = $this->_initMergerDir('js');
		if (!$targetDir) {
			return '';
		}

		$linkMerge 	= $this->_helper()->getMediaMergeUrl('js' . DS . $targetFilename, $this->getStore());
		$localFiles	= $this->_removeParamFiles($files, 'js');
		if(!empty($linkMerge)){
			if ($this->_mergeFiles($localFiles, $targetDir . DS . $targetFilename, false, null, 'js')) {
				$version = $this->_helper()->_getFileVersion('js' . DS . $targetFilename);
				return $linkMerge . 'js/' . $targetFilename . $version;
			}
		}

		return '';
	}

	public function getSkinBaseMageUrl(array $params=array())
	{
		$params['_type'] = 'skin';
		$this->updateParamDefaults($params);
		$baseUrl = Mage::getBaseUrl(Onlinebiz_Optimize_Model_Core_Store::URL_TYPE_SKIN_BASE, isset($params['_secure'])?(bool)$params['_secure']:null)
		.$params['_area'].'/'.$params['_package'].'/'.$params['_theme'].'/';
		return $baseUrl;
	}

	private function _removeParamFiles( $files, $type = 'css' )
	{
		$localFiles	= array();
		$storeId	= Mage::app()->getStore()->getId();
		$baseWebUrl	= Mage::getBaseUrl(Onlinebiz_Optimize_Model_Core_Store::URL_TYPE_WEB);
		if($this->isCloud()){
			$baseWebUrl = str_replace( '{{type}}', 'raw', $this->_helper()->getCloudinaryConfigData('url',$storeId) );
			if($this->_helper()->getCssJsCloudUrl()){
				$baseWebUrl = $this->_helper()->getCssJsCloudUrl();
			}
		}

		$useVersion    = $this->_helper()->isFileVersion() ;
		$baseMediaDir = Mage::getBaseDir() . DS;
		if(is_array($files)){
			foreach ($files as $file) {
				$newFile = $file;
				if( $useVersion ){
					$fileInfo  = parse_url($file);
					if( isset($fileInfo['query'])){
						$newFile = str_replace( '?'. $fileInfo['query'],'', $newFile);
					}
				}
				$localFiles[] = str_replace( $baseWebUrl, $baseMediaDir, $newFile );
			}
		}
		return $localFiles;
	}


	/**
	* Prepare url for css replacement
	*
	* @param string $uri
	* @return string
	*/
	protected function _prepareUrl($uri)
	{
		// check absolute or relative url
		if (!preg_match('/^https?:/i', $uri) && !preg_match('/^\//i', $uri)) {
			$fileDir = '';
			$pathParts = explode(DS, $uri);
			$fileDirParts = explode(DS, $this->_callbackFileDir);

			$store = $this->getStore();
			$isCssCompression = $this->_helper()->isUseCompression('css',$store);

			if($isCssCompression && !$store->isAdmin()){
				array_unshift($fileDirParts, 'skin');
			}

			if (is_int($store)) {
				$store = Mage::app()->getStore($store);
			}
			if ($store->isAdmin()) {
				$secure = $store->isAdminUrlSecure();
			} else {
				$secure = $store->isFrontUrlSecure() && Mage::app()->getRequest()->isSecure();
			}

			if ('skin' == $fileDirParts[0]) {
				$baseUrl = Mage::getBaseUrl('skin', $secure);
				$fileDirParts = array_slice($fileDirParts, 1);
			} elseif ('media' == $fileDirParts[0]) {
				$baseUrl = Mage::getBaseUrl('media', $secure);
				$fileDirParts = array_slice($fileDirParts, 1);
			} else {
				$baseUrl = Mage::getBaseUrl('web', $secure);
			}

			foreach ($pathParts as $key=>$part) {
				if ($part == '.' || $part == '..') {
					unset($pathParts[$key]);
				}
				if ($part == '..' && count($fileDirParts)) {
					$fileDirParts = array_slice($fileDirParts, 0, count($fileDirParts) - 1);
				}
			}

			if (count($fileDirParts)) {
				$fileDir = implode('/', $fileDirParts).'/';
			}
			if( $fileDir == 'media/css_secure/' || $fileDir == '/media/css_secure/')
			$fileDir = '';
			$pathParts = implode('/', $pathParts);
			if($isCssCompression){
				$pathParts = str_replace('../../skin/','',$pathParts);
				$pathParts = str_replace('skin/','',$pathParts);
			}
			if( $this->_helper()->isFontsFile($pathParts) && $this->isCloud()){
				$baseUrl = Mage::getBaseUrl('skincloud', $secure);
			}
			$uri = $baseUrl . $fileDir . $pathParts;
		}
		return $uri;
	}


	/**
	* Callback function replaces relative links for @import matches in css file
	*
	* @param array $match
	* @return string
	*/
	protected function _cssMergerImportCallback($match)
	{
		$quote = $match[1];
		$uri = $this->_prepareUrl($match[2]);

		return "@import {$quote}{$uri}{$quote}";
	}

	/**
	* Callback function replaces relative links for url() matches in css file
	*
	* @param array $match
	* @return string
	*/
	protected function _cssMergerUrlCallback($match)
	{
		$quote = ($match[1][0] == "'" || $match[1][0] == '"') ? $match[1][0] : '';
		$uri = ($quote == '') ? $match[1] : substr($match[1], 1, strlen($match[1]) - 2);
		$uri = $this->_prepareUrl($uri);
		if($this->isCloud() && !$quote)
		return "url('{$uri}')";
		return "url({$quote}{$uri}{$quote})";
	}
}
